"""SSH helpers"""
import asyncio
import logging

# STM-9317 suppress warnings until STM-5490 is fixed.
import warnings
from cryptography.utils import CryptographyDeprecationWarning

warnings.filterwarnings("ignore", category=CryptographyDeprecationWarning)

import asyncssh

asyncssh.set_log_level(logging.ERROR)


class SshClient():
    """Helper class for a SSH client.  User is responsible for retry/errors."""

    def __init__(self,
                 host,
                 port=None,
                 username=None,
                 password=None,
                 tunnel=None,
                 init_wait=0,
                 verbose=0):
        self.host = host
        self.port = port or 22
        self.username = username
        self.password = password
        self.init_wait = init_wait
        self._tun = tunnel

        self.verbose = verbose
        self._conn = None
        self.stdin = self.stdout = self.stderr = None

    async def _open(self):
        if self._conn is None:
            try:
                self._conn = await asyncio.wait_for(asyncssh.connect(
                    self.host,
                    port=self.port,
                    known_hosts=None,
                    tunnel=self._tun,
                    username=self.username,
                    password=self.password),
                                                    timeout=5.0)
            except Exception as err:
                logging.warning('Error connecting to %s: %s', self.host, err)
                raise

    async def close(self):
        """Close the connection and free any resources."""
        if self._conn:
            self._conn.close()
            self._conn = None

    async def tunnel(self, host, port=None, username=None, password=None):
        """Create a new client tunnelled through this one."""
        if self._conn is None:
            await self._open()
        return SshClient(host,
                         port,
                         username=username,
                         password=password,
                         tunnel=self._conn,
                         verbose=self.verbose)

    async def cmd(self, cmd, timeout=5.0, delimiter=None, term_type=None):
        """Send a command.  The response is returned."""
        if self._conn is None:
            await self._open()
        if self.stdin is None:
            try:
                self.stdin, self.stdout, self.stderr = await self._conn.open_session(
                    term_type=term_type)
                if not delimiter:
                    # this sometimes helps "kick" a connection so the first
                    # command works when it wouldn't otherwise.  saw this with
                    # some ignitenet dropbear servers.  only done when
                    # the default delimiter is used because it can break the
                    # regular delimiter.
                    self.stdin.write('\n')
                    await asyncio.sleep(self.init_wait)
                    await self.stdout.read(2048)
                else:
                    r = await asyncio.wait_for(self.stdout.readuntil(delimiter),
                                               timeout=timeout)
            except asyncio.CancelledError:
                raise
            except Exception as err:
                logging.warning('Error establishing session to %s: %s'.format(
                    self.host, err))
                raise

        if self.verbose > 2:
            print('#!>{}'.format(cmd))
        token = delimiter or '--PRESEEM--'
        if delimiter is None:  # default behavior is we add a token to output
            cmd += '; echo -n {}\n'.format(token)
        self.stdin.write(cmd)
        r = await asyncio.wait_for(self.stdout.readuntil(token), timeout=timeout)
        if not delimiter:
            r = r[:-11]
        if self.verbose > 2:
            print('#!<{}'.format(r))
        return r
